@@ -15,6 +15,7 @@
 import logging
 import os
 import random
+import string
 from typing import List, Optional, Tuple, cast
 
 import click
@@ -27,11 +28,14 @@ from aws_orbit.models.manifest import (
     DataNetworkingManifest,
     FoundationManifest,
     ImageManifest,
+    ImagesManifest,
     Manifest,
     ManifestSerDe,
     NetworkingManifest,
 )
 from aws_orbit.services import cfn, codebuild
+from aws_orbit.services import cognito as orbit_cognito
+from aws_orbit.services import ssm
 
 _logger: logging.Logger = logging.getLogger(__name__)
 
@@ -39,9 +43,7 @@ _logger: logging.Logger = logging.getLogger(__name__)
 def _request_dockerhub_credential(msg_ctx: MessagesContext) -> Tuple[str, str]:
     if msg_ctx.pbar is not None:
         msg_ctx.pbar.clear()
-    msg_ctx.info(
-        "When Container Images are built from source or pulled from Dockerhub, " "a Dockerhub login is required."
-    )
+    msg_ctx.info("When Container Images are from Dockerhub, " "a Dockerhub login is required.")
     username = cast(
         str,
         click.prompt("Please enter the DockerHub username", type=str, hide_input=False),
@@ -59,11 +61,10 @@ def _get_images_dirs(context: "Context", manifest_filename: str, skip_images: bo
     else:
         refdir: str = os.path.dirname(os.path.abspath(manifest_filename))
         _logger.debug("refdir: %s", refdir)
-        _logger.debug(context.images.jupyter_hub.source)
         dirs = [
             (os.path.join(refdir, getattr(context.images, name).path), name.replace("_", "-"))
             for name in context.images.names
-            if getattr(context.images, name).source == "code"
+            if getattr(context.images, name).get_source(account_id=context.account_id, region=context.region) == "code"
         ]
         _logger.debug("dirs: %s", dirs)
     return dirs
@@ -88,7 +89,9 @@ def deploy_toolkit(
     stack_exist: bool = cfn.does_stack_exist(stack_name=context.toolkit.stack_name)
     credential_exist: bool = dockerhub.does_credential_exist(context=context) if stack_exist else False
     image_manifests = [cast(ImageManifest, getattr(context.images, i)) for i in context.images.names]
-    credential_required: bool = any([im.source == "dockerhub" or im.source == "code" for im in image_manifests])
+    credential_required: bool = any(
+        [im.get_source(account_id=context.account_id, region=context.region) == "dockerhub" for im in image_manifests]
+    )
 
     if stack_exist:
         if credential_required and not credential_exist and not credential_received:
@@ -103,7 +106,7 @@ def deploy_toolkit(
             )
             credential_exist = True
     else:
-        context.toolkit.deploy_id = "%06x" % random.randrange(16 ** 6)
+        context.toolkit.deploy_id = "".join(random.choice(string.ascii_lowercase) for i in range(6))
         if credential_required and not credential_received:
             username, password = _request_dockerhub_credential(msg_ctx=msg_ctx)
             credential_exist = False
@@ -212,6 +215,15 @@ def deploy_env(
         msg_ctx.progress(3)
 
         context: "Context" = ContextSerDe.load_context_from_manifest(manifest=manifest)
+        image_manifests = {"code_build": manifest.images.code_build}
+
+        for name in context.images.names:
+            # We don't allow these images to be managed with an input Manifest
+            # These images should be changed/maintained in manifests.py
+            if name not in ["code_build", "image_replicator", "k8s-utilities"]:
+                image_manifests[name] = getattr(context.images, name) if skip_images else getattr(manifest.images, name)
+        context.images = ImagesManifest(**image_manifests)  # type: ignore
+
         msg_ctx.info("Current Context loaded")
         msg_ctx.progress(4)
 
@@ -263,7 +275,7 @@ def deploy_env(
         else:
             RuntimeError("Cognito Users URL not found.")
         if context.landing_page_url:
-            msg_ctx.tip(f"Access Orbit Workbench: {stylize(context.landing_page_url, underline=True)}")
+            msg_ctx.tip(f"Access Orbit Workbench: {stylize(f'{context.landing_page_url}/orbit/login', underline=True)}")
         else:
             RuntimeError("Landing Page URL not found.")
         msg_ctx.progress(100)
@@ -281,6 +293,11 @@ def deploy_teams(
         msg_ctx.info(f"Teams: {','.join([t.name for t in manifest.teams])}")
         msg_ctx.progress(3)
 
+        context_parameter_name: str = f"/orbit/{manifest.name}/context"
+        if not ssm.does_parameter_exist(name=context_parameter_name):
+            msg_ctx.error(f"Orbit Environment {manifest.name} cannot be found in the current account and region.")
+            return
+
         context: "Context" = ContextSerDe.load_context_from_manifest(manifest=manifest)
         msg_ctx.info("Current Context loaded")
         msg_ctx.info(f"Teams: {','.join([t.name for t in context.teams])}")
@@ -329,18 +346,18 @@ def deploy_teams(
         msg_ctx.progress(98)
 
         if cfn.does_stack_exist(stack_name=context.env_stack_name):
-            context = ContextSerDe.load_context_from_manifest(manifest=manifest)
+            context = ContextSerDe.load_context_from_ssm(env_name=manifest.name, type=Context)
             msg_ctx.info(f"Context updated: {filename}")
         msg_ctx.progress(99)
 
-        if context.cognito_users_url:
-            msg_ctx.tip(f"Add users: {stylize(context.cognito_users_url, underline=True)}")
-        else:
-            RuntimeError("Cognito Users URL not found.")
+        if context.user_pool_id:
+            cognito_users_url = orbit_cognito.get_users_url(user_pool_id=context.user_pool_id, region=context.region)
+            msg_ctx.tip(f"Add users: {stylize(cognito_users_url, underline=True)}")
+
         if context.landing_page_url:
-            msg_ctx.tip(f"Access Orbit Workbench: {stylize(context.landing_page_url, underline=True)}")
+            msg_ctx.tip(f"Access Orbit Workbench: {stylize(f'{context.landing_page_url}/orbit/login', underline=True)}")
         else:
-            RuntimeError("Landing Page URL not found.")
+            raise RuntimeError("Landing Page URL not found.")
         msg_ctx.progress(100)
 
 
@@ -357,7 +374,7 @@ def _deploy_image(
         context: "Context" = ContextSerDe.load_context_from_ssm(env_name=env, type=Context)
 
         if cfn.does_stack_exist(stack_name=f"orbit-{context.name}") is False:
-            msg_ctx.error("Please, deploy your environment before deploy any addicional docker image")
+            msg_ctx.error("Please, deploy your environment before deploy any additional docker image")
             return
 
         plugins.PLUGINS_REGISTRIES.load_plugins(
