@@ -25,6 +25,7 @@ from dataclasses import field
 from marshmallow import Schema
 from marshmallow_dataclass import dataclass
 
+import aws_orbit
 from aws_orbit import utils
 from aws_orbit.models.common import BaseSchema
 from aws_orbit.models.manifest import (
@@ -153,6 +154,7 @@ class TeamContext:
     container_defaults: Dict[str, Any] = field(default_factory=get_container_defaults)
     efs_id: Optional[str] = None
     efs_ap_id: Optional[str] = None
+    efs_private_ap_id: Optional[str] = None
     eks_pod_role_arn: Optional[str] = None
     jupyter_url: Optional[str] = None
     ecs_cluster_name: Optional[str] = None
@@ -164,12 +166,15 @@ class TeamContext:
     # added with defaults for backward compatability
     fargate: bool = True
     k8_admin: bool = False
+    team_helm_repository: Optional[str] = None
+    user_helm_repository: Optional[str] = None
 
     def fetch_team_data(self) -> None:
         _logger.debug("Fetching Team %s data...", self.name)
         values = ssm.get_parameter(name=self.team_ssm_parameter_name)
         self.efs_id = values["EfsId"]
         self.efs_ap_id = values["EfsApId"]
+        self.efs_private_ap_id = values["EfsApIdPrivate"]
         self.eks_pod_role_arn = values["EksPodRoleArn"]
         self.scratch_bucket = values["ScratchBucket"]
         self.team_kms_key_arn = values["TeamKmsKeyArn"]
@@ -214,6 +219,7 @@ class FoundationContext:
     codeartifact_repository: Optional[str] = None
     images: FoundationImagesManifest = FoundationImagesManifest()
     policies: Optional[List[str]] = cast(List[str], field(default_factory=list))
+    orbit_version: Optional[str] = aws_orbit.__version__
 
 
 @dataclass(base_schema=BaseSchema)
@@ -258,6 +264,10 @@ class Context:
     cluster_pod_sg_id: Optional[str] = None
     managed_nodegroups: List[ManagedNodeGroupManifest] = field(default_factory=list)
     policies: Optional[List[str]] = cast(List[str], field(default_factory=list))
+    orbit_version: Optional[str] = aws_orbit.__version__
+    helm_repository: Optional[str] = None
+    install_ssm_agent: Optional[bool] = False
+    install_image_replicator: Optional[bool] = False
 
     def get_team_by_name(self, name: str) -> Optional[TeamContext]:
         for t in self.teams:
@@ -306,7 +316,12 @@ def create_team_context_from_manifest(manifest: "Manifest", team_manifest: "Team
     region: str = utils.get_region()
     ssm_parameter_name: str = f"/orbit/{manifest.name}/teams/{team_manifest.name}/context"
     version = manifest.images.jupyter_user.version
-    base_image_address = f"{account_id}.dkr.ecr.{region}.amazonaws.com/orbit-{manifest.name}-jupyter-user:{version}"
+    _logger.debug("Team Base Image: %s - %s", team_manifest.name, manifest.images.jupyter_user)
+    base_image_address = (
+        f"{account_id}.dkr.ecr.{region}.amazonaws.com/orbit-{manifest.name}/jupyter-user:{version}"
+        if manifest.images.jupyter_user.get_source(account_id=account_id, region=region) == "code"
+        else f"{manifest.images.jupyter_user.repository}:{version}"
+    )
     final_image_address = base_image_address
     return TeamContext(
         base_image_address=base_image_address,
@@ -371,13 +386,16 @@ class ContextSerDe(Generic[T, V]):
         context_parameter_name: str = f"/orbit/{manifest.name}/context"
         if ssm.does_parameter_exist(name=context_parameter_name):
             context: Context = ContextSerDe.load_context_from_ssm(env_name=manifest.name, type=Context)
-            context.images = manifest.images
             context.networking = create_networking_context_from_manifest(networking=manifest.networking)
             context.user_pool_id = manifest.user_pool_id
             context.shared_efs_fs_id = manifest.shared_efs_fs_id
             context.shared_efs_sg_id = manifest.shared_efs_sg_id
             context.scratch_bucket_arn = manifest.scratch_bucket_arn
             context.policies = manifest.policies
+            context.codeartifact_domain = manifest.codeartifact_domain
+            context.codeartifact_repository = manifest.codeartifact_repository
+            context.cognito_external_provider = manifest.cognito_external_provider
+            context.cognito_external_provider_label = manifest.cognito_external_provider_label
             for team_manifest in manifest.teams:
                 team_context: Optional[TeamContext] = context.get_team_by_name(name=team_manifest.name)
                 if team_context:
@@ -412,7 +430,12 @@ class ContextSerDe(Generic[T, V]):
                 managed_nodegroups=[],
                 eks_system_masters_roles=[],
                 policies=manifest.policies,
+                install_ssm_agent=manifest.install_ssm_agent,
+                install_image_replicator=manifest.install_image_replicator,
             )
+        context.install_ssm_agent = manifest.install_ssm_agent
+        context.install_image_replicator = manifest.install_image_replicator
+
         ContextSerDe.fetch_toolkit_data(context=context)
         ContextSerDe.dump_context_to_ssm(context=context)
         return context
@@ -427,6 +450,8 @@ class ContextSerDe(Generic[T, V]):
             )
             context.images = manifest.images
             context.policies = manifest.policies
+            context.codeartifact_domain = manifest.codeartifact_domain
+            context.codeartifact_repository = manifest.codeartifact_repository
         else:
             context = FoundationContext(
                 name=manifest.name,
@@ -457,6 +482,8 @@ class ContextSerDe(Generic[T, V]):
             content: Dict[str, Any] = cast(Dict[str, Any], Context.Schema().dump(context))
         elif isinstance(context, FoundationContext):
             content = cast(Dict[str, Any], FoundationContext.Schema().dump(context))
+        elif isinstance(context, TeamContext):
+            content = cast(Dict[str, Any], TeamContext.Schema().dump(context))
         else:
             raise ValueError("Unknown 'context' Type")
         return str(json.dumps(obj=content, sort_keys=True))
@@ -484,6 +511,7 @@ class ContextSerDe(Generic[T, V]):
             raise ValueError("Unknown 'context' Type")
 
         ssm.put_parameter(name=ssm_parameter_name, obj=content)
+        _logger.debug("Context written to SSM: %s", content)
 
     @staticmethod
     def load_context_from_ssm(env_name: str, type: Type[V]) -> V:
@@ -495,15 +523,16 @@ class ContextSerDe(Generic[T, V]):
                 msg = f"SSM parameter {context_parameter_name} not found for env {env_name}"
                 _logger.error(msg)
                 raise Exception(msg)
+            _logger.debug("Raw SSM: %s", main)
             teams_parameters = ssm.list_parameters(prefix=f"/orbit/{env_name}/teams/")
             _logger.debug("teams_parameters: %s", teams_parameters)
             teams = [ssm.get_parameter_if_exists(name=p) for p in teams_parameters if p.endswith("/context")]
             main["Teams"] = [t for t in teams if t]
-            return cast(V, Context.Schema().load(data=main, many=False, partial=False, unknown="RAISE"))
+            return cast(V, Context.Schema().load(data=main, many=False, partial=False, unknown="EXCLUDE"))
         elif type is FoundationContext:
             context_parameter_name = f"/orbit-foundation/{env_name}/context"
             main = ssm.get_parameter(name=context_parameter_name)
-            return cast(V, FoundationContext.Schema().load(data=main, many=False, partial=False, unknown="RAISE"))
+            return cast(V, FoundationContext.Schema().load(data=main, many=False, partial=False, unknown="EXCLUDE"))
         else:
             raise ValueError("Unknown 'context' Type")
 
@@ -560,4 +589,12 @@ class ContextSerDe(Generic[T, V]):
         context.cdk_toolkit.s3_bucket = (
             f"{top_level}-{context.name}-cdk-toolkit-{context.account_id}-{context.toolkit.deploy_id}"
         )
+
+        if isinstance(context, Context):
+            # Set the Helm repositories
+            _logger.debug(f"context.helm_repository: s3://{context.toolkit.s3_bucket}/helm/repositories/env")
+            context.helm_repository = f"s3://{context.toolkit.s3_bucket}/helm/repositories/env"
+
+            _logger.debug("context.toolkit: %s", context.toolkit)
+
         _logger.debug("Toolkit data fetched successfully.")
