@@ -23,7 +23,7 @@ from typing import Any, ClassVar, Dict, Generic, List, Optional, Set, Type, Type
 import jsonpath_ng as jsonpath_ng
 import yaml
 from dataclasses import field
-from marshmallow import Schema
+from marshmallow import EXCLUDE, Schema
 from marshmallow_dataclass import dataclass
 from yamlinclude import YamlIncludeConstructor
 
@@ -66,10 +66,25 @@ class TeamManifest:
 class ImageManifest:
     Schema: ClassVar[Type[Schema]] = Schema
     repository: Optional[str]
-    source: Optional[str] = "ecr-public"
-    version: Optional[str] = aws_orbit.__version__
+    version: Optional[str] = "latest" if aws_orbit.__version__.endswith(".dev0") else aws_orbit.__version__
     path: Optional[str] = None
 
+    def get_source(self, account_id: str, region: str) -> str:
+        external_ecr_match = re.compile(r"^[0-9]{12}\.dkr\.ecr\..+\.amazonaws.com/")
+        public_ecr_match = re.compile(r"^public.ecr.aws/.+/")
+        repository = cast(str, self.repository)
+
+        if self.path:
+            return "code"
+        elif repository.startswith(f"{account_id}.dkr.ecr.{region}.amazonaws.com"):
+            return "ecr-internal"
+        elif external_ecr_match.match(repository):
+            return "ecr-external"
+        elif public_ecr_match.match(repository):
+            return "ecr-public"
+        else:
+            return "public"
+
 
 @dataclass(base_schema=BaseSchema, frozen=True)
 class ManagedNodeGroupManifest:
@@ -87,11 +102,7 @@ class ManagedNodeGroupManifest:
 @dataclass(base_schema=BaseSchema, frozen=True)
 class CodeBuildImageManifest(ImageManifest):
     repository: Optional[str] = "public.ecr.aws/v3o4w1g6/aws-orbit-workbench/code-build-base"
-
-
-@dataclass(base_schema=BaseSchema, frozen=True)
-class JupyterHubImageManifest(ImageManifest):
-    repository: Optional[str] = "public.ecr.aws/v3o4w1g6/aws-orbit-workbench/jupyter-hub"
+    version: Optional[str] = "1.0.0"
 
 
 @dataclass(base_schema=BaseSchema, frozen=True)
@@ -100,56 +111,26 @@ class JupyterUserImageManifest(ImageManifest):
 
 
 @dataclass(base_schema=BaseSchema, frozen=True)
-class LandingPageImageManifest(ImageManifest):
-    repository: Optional[str] = "public.ecr.aws/v3o4w1g6/aws-orbit-workbench/landing-page"
+class OrbitControllerImageManifest(ImageManifest):
+    repository: Optional[str] = "public.ecr.aws/v3o4w1g6/aws-orbit-workbench/orbit-controller"
 
 
 @dataclass(base_schema=BaseSchema, frozen=True)
-class AwsEfsDriverImageManifest(ImageManifest):
-    repository: Optional[str] = "602401143452.dkr.ecr.us-west-2.amazonaws.com/eks/aws-efs-csi-driver"
-    source: Optional[str] = "ecr-external"
-    version: Optional[str] = "v1.0.0"
+class ImageReplicatorImageManifest(ImageManifest):
+    repository: Optional[str] = "public.ecr.aws/v3o4w1g6/aws-orbit-workbench/image-replicator"
+    version: Optional[str] = "1.0.0"
 
 
 @dataclass(base_schema=BaseSchema, frozen=True)
-class LivenessprobeImageManifest(ImageManifest):
-    repository: Optional[str] = "602401143452.dkr.ecr.us-west-2.amazonaws.com/eks/livenessprobe"
-    source: Optional[str] = "ecr-external"
-    version: Optional[str] = "v2.0.0"
+class K8sUtilitiesImageManifest(ImageManifest):
+    repository: Optional[str] = "public.ecr.aws/v3o4w1g6/aws-orbit-workbench/k8s-utilities"
+    version: Optional[str] = "1.0.0"
 
 
 @dataclass(base_schema=BaseSchema, frozen=True)
-class CsiNodeDriverRegistrarImageManifest(ImageManifest):
-    repository: Optional[str] = "602401143452.dkr.ecr.us-west-2.amazonaws.com/eks/csi-node-driver-registrar"
-    source: Optional[str] = "ecr-external"
-    version: Optional[str] = "v1.3.0"
-
-
-# https://github.com/kubernetes/dashboard/releases
-@dataclass(base_schema=BaseSchema, frozen=True)
-class K8Dashboard(ImageManifest):
-    repository: Optional[str] = "public.ecr.aws/v3o4w1g6/aws-orbit-workbench/kubernetesui/dashboard"
-    version: Optional[str] = "v2.2.0"
-
-
-@dataclass(base_schema=BaseSchema, frozen=True)
-class MetricsScraper(ImageManifest):
-    repository: Optional[str] = "public.ecr.aws/v3o4w1g6/aws-orbit-workbench/kubernetesui/metrics-scraper"
-    version: Optional[str] = "v1.0.6"
-
-
-# https://github.com/kubernetes-sigs/metrics-server/releases
-@dataclass(base_schema=BaseSchema, frozen=True)
-class MetricsServer(ImageManifest):
-    repository: Optional[str] = "public.ecr.aws/v3o4w1g6/aws-orbit-workbench/k8s.gcr.io/metrics-server/metrics-server"
-    version: Optional[str] = "v0.4.2"
-
-
-# https://github.com/kubernetes/autoscaler/tree/master/cluster-autoscaler/cloudprovider/aws
-@dataclass(base_schema=BaseSchema, frozen=True)
-class ClusterAutoscaler(ImageManifest):
-    repository: Optional[str] = "public.ecr.aws/v3o4w1g6/aws-orbit-workbench/k8s.gcr.io/autoscaling/cluster-autoscaler"
-    version: Optional[str] = "v1.18.3"
+class UtilityDataImageManifest(ImageManifest):
+    repository: Optional[str] = "public.ecr.aws/v3o4w1g6/aws-orbit-workbench/utility-data"
+    version: Optional[str] = "1.0.0"
 
 
 @dataclass(base_schema=BaseSchema, frozen=True)
@@ -167,31 +148,21 @@ class FoundationImagesManifest:
 @dataclass(base_schema=BaseSchema, frozen=True)
 class ImagesManifest:
     Schema: ClassVar[Type[Schema]] = Schema
-    code_build: CodeBuildImageManifest = CodeBuildImageManifest()
-    jupyter_hub: JupyterHubImageManifest = JupyterHubImageManifest()
-    jupyter_user: JupyterUserImageManifest = JupyterUserImageManifest()
-    landing_page: LandingPageImageManifest = LandingPageImageManifest()
-    aws_efs_csi_driver: AwsEfsDriverImageManifest = AwsEfsDriverImageManifest()
-    livenessprobe: LivenessprobeImageManifest = LivenessprobeImageManifest()
-    csi_node_driver_registrar: CsiNodeDriverRegistrarImageManifest = CsiNodeDriverRegistrarImageManifest()
-    k8_dashboard: K8Dashboard = K8Dashboard()
-    k8_metrics_scraper: MetricsScraper = MetricsScraper()
-    k8_metrics_server: MetricsServer = MetricsServer()
-    cluster_autoscaler: ClusterAutoscaler = ClusterAutoscaler()
+    code_build: ImageManifest = CodeBuildImageManifest()
+    jupyter_user: ImageManifest = JupyterUserImageManifest()
+    orbit_controller: ImageManifest = OrbitControllerImageManifest()
+    image_replicator: ImageManifest = ImageReplicatorImageManifest()
+    k8s_utilities: ImageManifest = K8sUtilitiesImageManifest()
+    utility_data: ImageManifest = UtilityDataImageManifest()
     names: List[str] = field(
         metadata=dict(load_only=True),
         default_factory=lambda: [
             "code_build",
-            "jupyter_hub",
             "jupyter_user",
-            "landing_page",
-            "aws_efs_csi_driver",
-            "livenessprobe",
-            "csi_node_driver_registrar",
-            "k8_dashboard",
-            "k8_metrics_scraper",
-            "k8_metrics_server",
-            "cluster_autoscaler",
+            "orbit_controller",
+            "image_replicator",
+            "k8s_utilities",
+            "utility_data",
         ],
     )
 
@@ -252,6 +223,8 @@ class Manifest:
     managed_nodegroups: List[ManagedNodeGroupManifest] = field(default_factory=list)
     policies: Optional[List[str]] = cast(List[str], field(default_factory=list))
     ssm_parameter_name: Optional[str] = None
+    install_ssm_agent: Optional[bool] = False
+    install_image_replicator: Optional[bool] = False
 
     def get_team_by_name(self, name: str) -> Optional[TeamManifest]:
         for t in self.teams:
@@ -364,6 +337,8 @@ def _add_env_var_injector(tag: str = "!ENV") -> None:
                 (env_var, default_val) = g.split("::")
                 value = os.environ.get(env_var, default_val)
                 full_value = full_value.replace(f"${{{g}}}", value)
+                if not full_value:
+                    full_value = None
                 _logger.debug(f"injected ENV parameter {env_var} resolved to {value}")
             return full_value
         return value
@@ -390,10 +365,10 @@ class ManifestSerDe(Generic[T]):
         _logger.debug("raw: %s", raw)
         if type is Manifest:
             raw["SsmParameterName"] = f"/orbit/{raw['Name']}/manifest"
-            manifest: T = cast(T, Manifest.Schema().load(data=raw, many=False, partial=False, unknown="RAISE"))
+            manifest: T = cast(T, Manifest.Schema().load(data=raw, many=False, partial=False, unknown=EXCLUDE))
         elif type is FoundationManifest:
             raw["SsmParameterName"] = f"/orbit-foundation/{raw['Name']}/manifest"
-            manifest = cast(T, FoundationManifest.Schema().load(data=raw, many=False, partial=False, unknown="RAISE"))
+            manifest = cast(T, FoundationManifest.Schema().load(data=raw, many=False, partial=False, unknown=EXCLUDE))
         else:
             raise ValueError("Unknown 'manifest' Type")
         ManifestSerDe.dump_manifest_to_ssm(manifest=manifest)
@@ -446,12 +421,12 @@ class ManifestSerDe(Generic[T]):
             _logger.debug("teams_parameters (/orbit/%s/teams/): %s", env_name, teams_parameters)
             teams = [ssm.get_parameter(name=p) for p in teams_parameters if p.endswith("/manifest")]
             main["Teams"] = teams
-            return cast(T, Manifest.Schema().load(data=main, many=False, partial=False, unknown="RAISE"))
+            return cast(T, Manifest.Schema().load(data=main, many=False, partial=False, unknown=EXCLUDE))
         elif type is FoundationManifest:
             context_parameter_name = f"/orbit-foundation/{env_name}/manifest"
             main = ssm.get_parameter_if_exists(name=context_parameter_name)
             if main is None:
                 return None
-            return cast(T, FoundationManifest.Schema().load(data=main, many=False, partial=False, unknown="RAISE"))
+            return cast(T, FoundationManifest.Schema().load(data=main, many=False, partial=False, unknown=EXCLUDE))
         else:
             raise ValueError("Unknown 'manifest' Type")
