@@ -13,7 +13,7 @@
 #    limitations under the License.
 
 import logging
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, cast
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Sequence, cast
 
 import aws_cdk.aws_iam as iam
 import aws_cdk.aws_kms as kms
@@ -38,6 +38,7 @@ class IamBuilder:
         policy_names: List[str],
         scratch_bucket: s3.IBucket,
         team_kms_key: kms.Key,
+        session_timeout: core.Duration,
     ) -> iam.Role:
         env_name = context.name
         partition = core.Aws.PARTITION
@@ -72,7 +73,10 @@ class IamBuilder:
                     actions=["s3:List*", "s3:Get*"],
                     resources=[
                         f"arn:{partition}:s3:::{context.toolkit.s3_bucket}",
-                        f"arn:{partition}:s3:::{context.toolkit.s3_bucket}/*",
+                        f"arn:{partition}:s3:::{context.toolkit.s3_bucket}/samples/*",
+                        f"arn:{partition}:s3:::{context.toolkit.s3_bucket}/teams/{team_name}/*",
+                        f"arn:{partition}:s3:::{context.toolkit.s3_bucket}/helm/repositories/env/*",
+                        f"arn:{partition}:s3:::{context.toolkit.s3_bucket}/helm/repositories/teams/{team_name}/*",
                     ],
                 ),
                 iam.PolicyStatement(
@@ -98,11 +102,11 @@ class IamBuilder:
                         "redshift:DeleteCluster",
                     ],
                     resources=[
-                        f"arn:{partition}:redshift:{region}:{account}:dbuser:{env_name}-{team_name}*",
-                        f"arn:{partition}:redshift:{region}:{account}:dbuser:{env_name}-{team_name}*/master",
-                        f"arn:{partition}:redshift:{region}:{account}:dbuser:{env_name}-{team_name}*/defaultdb",
-                        f"arn:{partition}:redshift:{region}:{account}:dbname:{env_name}-{team_name}*/defaultdb",
-                        f"arn:{partition}:redshift:{region}:{account}:cluster:{env_name}-{team_name}*",
+                        f"arn:{partition}:redshift:{region}:{account}:dbuser:orbit-{env_name}-{team_name}*",
+                        f"arn:{partition}:redshift:{region}:{account}:dbuser:orbit-{env_name}-{team_name}*/master",
+                        f"arn:{partition}:redshift:{region}:{account}:dbuser:orbit-{env_name}-{team_name}*/defaultdb",
+                        f"arn:{partition}:redshift:{region}:{account}:dbname:orbit-{env_name}-{team_name}*/defaultdb",
+                        f"arn:{partition}:redshift:{region}:{account}:cluster:orbit-{env_name}-{team_name}*",
                     ],
                 ),
                 iam.PolicyStatement(
@@ -156,37 +160,11 @@ class IamBuilder:
                         "elasticmapreduce:Describe*",
                         "elasticmapreduce:TerminateJobFlows",
                         "elasticmapreduce:AddJobFlowSteps",
-                        "sagemaker:List*",
-                        "sagemaker:Get*",
-                        "sagemaker:Describe*",
-                        "sagemaker:CreateModel",
-                        "sagemaker:DeleteModelPackage",
-                        "sagemaker:UpdateEndpointWeightsAndCapacities",
-                        "sagemaker:DeleteAlgorithm",
-                        "sagemaker:Search",
-                        "sagemaker:UpdateWorkteam",
-                        "sagemaker:DeleteModel",
-                        "sagemaker:CreateModelPackage",
-                        "sagemaker:DeleteWorkteam",
-                        "sagemaker:CreateEndpoint",
-                        "sagemaker:CreateEndpointConfig",
-                        "sagemaker:RenderUiTemplate",
-                        "sagemaker:StopTransformJob",
-                        "sagemaker:CreateLabelingJob",
-                        "sagemaker:DeleteEndpointConfig",
-                        "sagemaker:CreateAlgorithm",
-                        "sagemaker:CreateTrainingJob",
-                        "sagemaker:StopHyperParameterTuningJob",
-                        "sagemaker:DeleteEndpoint",
-                        "sagemaker:CreateTransformJob",
-                        "sagemaker:InvokeEndpoint",
-                        "sagemaker:CreateWorkteam",
-                        "sagemaker:StopLabelingJob",
-                        "sagemaker:UpdateEndpoint",
-                        "sagemaker:CreateCompilationJob",
-                        "sagemaker:StopCompilationJob",
-                        "sagemaker:CreateHyperParameterTuningJob",
+                        "sagemaker:*",
+                        "databrew:*",
                         "lakeformation:GetDataAccess",
+                        "fsx:Describe*",
+                        "fsx:List*",
                     ],
                     resources=["*"],
                 ),
@@ -206,6 +184,7 @@ class IamBuilder:
                         f"arn:{partition}:logs:{region}:{account}:log-group:/aws/sagemaker/*:log-stream:*",
                         f"arn:{partition}:logs:{region}:{account}:log-group:/aws/eks/orbit*",
                         f"arn:{partition}:events:{region}:{account}:rule/orbit-{env_name}-{team_name}-*",
+                        f"arn:{partition}:logs:{region}:{account}:log-group:/aws-glue-databrew/*",
                     ],
                 ),
                 iam.PolicyStatement(
@@ -213,7 +192,7 @@ class IamBuilder:
                     actions=[
                         "ecr:*",
                     ],
-                    resources=[f"arn:{partition}:ecr:{region}:{account}:repository/orbit-{env_name}-users-*"],
+                    resources=[f"arn:{partition}:ecr:{region}:{account}:repository/orbit-{env_name}/users/*"],
                 ),
                 iam.PolicyStatement(
                     effect=iam.Effect.ALLOW,
@@ -228,6 +207,7 @@ class IamBuilder:
                     resources=[
                         f"arn:{partition}:lambda:{region}:{account}:function:orbit-{env_name}-{team_name}-*",
                         f"arn:{partition}:lambda:{region}:{account}:function:orbit-{env_name}-token-validation",
+                        f"arn:{partition}:lambda:{region}:{account}:function:orbit-{env_name}-eks-service-handler",
                     ],
                 ),
                 iam.PolicyStatement(
@@ -291,7 +271,8 @@ class IamBuilder:
                         "logs:PutLogEvents",
                     ],
                     resources=[
-                        f"arn:{partition}:logs:{region}:{account}:log-group:/aws/codebuild/orbit-{env_name}:log-stream:*"  # noqa
+                        f"arn:{partition}:logs:{region}:{account}:log-group:/aws/codebuild/orbit-{env_name}:log-stream:*",  # noqa
+                        f"arn:{partition}:logs:{region}:{account}:log-group:/aws-glue-databrew/*:log-stream:*",
                     ],
                 ),
                 iam.PolicyStatement(
@@ -307,6 +288,7 @@ class IamBuilder:
                     ],
                     resources=[
                         f"arn:{partition}:logs:{region}:{account}:log-group:/aws/codebuild/orbit-{env_name}*:log-stream:*",  # noqa
+                        f"arn:{partition}:logs:{region}:{account}:log-group:/aws-glue-databrew/*:log-stream:*",
                     ],
                 ),
                 iam.PolicyStatement(
@@ -318,6 +300,15 @@ class IamBuilder:
                         f"arn:{partition}:ecr:{region}:{account}:repository/*",
                     ],
                 ),
+                iam.PolicyStatement(
+                    effect=iam.Effect.ALLOW,
+                    actions=[
+                        "eks:DescribeCluster",
+                    ],
+                    resources=[
+                        f"arn:{partition}:eks:{region}:{account}:cluster/orbit-{env_name}",
+                    ],
+                ),
             ],
         )
 
@@ -340,21 +331,28 @@ class IamBuilder:
             if "orbit" in policy_name
         ]
 
-        managed_policies = managed_policies + aws_managed_user_policies + orbit_custom_policies
+        managed_policies = (
+            managed_policies + cast(List[object], aws_managed_user_policies) + cast(List[object], orbit_custom_policies)
+        )
 
         role = iam.Role(
             scope=scope,
             id=lake_role_name,
             role_name=lake_role_name,
-            assumed_by=iam.CompositePrincipal(
-                iam.ServicePrincipal("ec2.amazonaws.com"),
-                iam.ServicePrincipal("glue.amazonaws.com"),
-                iam.ServicePrincipal("sagemaker.amazonaws.com"),
-                iam.ServicePrincipal("redshift.amazonaws.com"),
-                iam.ServicePrincipal("codepipeline.amazonaws.com"),
-                iam.ServicePrincipal("personalize.amazonaws.com"),
+            assumed_by=cast(
+                iam.IPrincipal,
+                iam.CompositePrincipal(
+                    iam.ServicePrincipal("ec2.amazonaws.com"),
+                    iam.ServicePrincipal("glue.amazonaws.com"),
+                    iam.ServicePrincipal("sagemaker.amazonaws.com"),
+                    iam.ServicePrincipal("redshift.amazonaws.com"),
+                    iam.ServicePrincipal("codepipeline.amazonaws.com"),
+                    iam.ServicePrincipal("personalize.amazonaws.com"),
+                    iam.ServicePrincipal("databrew.amazonaws.com"),
+                ),
             ),
-            managed_policies=managed_policies,
+            managed_policies=cast(Optional[Sequence[iam.IManagedPolicy]], managed_policies),
+            max_session_duration=session_timeout,
         )
         if role.assume_role_policy:
             role.assume_role_policy.add_statements(
@@ -362,13 +360,16 @@ class IamBuilder:
                     effect=iam.Effect.ALLOW,
                     actions=["sts:AssumeRoleWithWebIdentity"],
                     principals=[
-                        iam.FederatedPrincipal(
-                            federated=f"arn:{partition}:iam::{account}:oidc-provider/{context.eks_oidc_provider}",
-                            conditions={
-                                "StringLike": {
-                                    f"{context.eks_oidc_provider}:sub": f"system:serviceaccount:{team_name}:*"
-                                }
-                            },
+                        cast(
+                            iam.IPrincipal,
+                            iam.FederatedPrincipal(
+                                federated=f"arn:{partition}:iam::{account}:oidc-provider/{context.eks_oidc_provider}",
+                                conditions={
+                                    "StringLike": {
+                                        f"{context.eks_oidc_provider}:sub": f"system:serviceaccount:{team_name}*:*"
+                                    }
+                                },
+                            ),
                         )
                     ],
                 ),
