@@ -106,7 +106,7 @@ def generate_manifest(context: "Context", name: str, nodegroups: Optional[List[M
     MANIFEST["iam"]["serviceRoleARN"] = context.eks_cluster_role_arn
     MANIFEST["managedNodeGroups"] = []
 
-    labels = {"orbit/node-group": "env", "orbit/usage": "reserved"}
+    labels = {"orbit/node-group": "env", "orbit/usage": "reserved", "orbit/node-type": "ec2"}
     tags = tags = {f"k8s.io/cluster-autoscaler/node-template/label/{k}": v for k, v in labels.items()}
     tags["Env"] = f"orbit-{context.name}"
 
@@ -225,10 +225,16 @@ def get_pod_to_cluster_rules(group_id: str) -> List[IpPermission]:
             user_id_group_pairs=[UserIdGroupPair(description="Kubernetes API from Pod", group_id=group_id)],
         ),
         IpPermission(
-            from_port=10250,
-            to_port=10250,
+            from_port=15000,
+            to_port=16000,
             ip_protocol="tcp",
-            user_id_group_pairs=[UserIdGroupPair(description="Kubelet from Pod", group_id=group_id)],
+            user_id_group_pairs=[UserIdGroupPair(description="Health Probes", group_id=group_id)],
+        ),
+        IpPermission(
+            from_port=9411,
+            to_port=9411,
+            ip_protocol="tcp",
+            user_id_group_pairs=[UserIdGroupPair(description="Zipkin", group_id=group_id)],
         ),
     ]
 
@@ -417,6 +423,34 @@ def deploy_env(context: "Context", changeset: Optional[Changeset]) -> None:
         },
     )
 
+    iam.add_assume_role_statement(
+        role_name=f"orbit-{context.name}-eks-cluster-role",
+        statement={
+            "Effect": "Allow",
+            "Principal": {"Federated": f"arn:aws:iam::{context.account_id}:oidc-provider/{context.eks_oidc_provider}"},
+            "Action": "sts:AssumeRoleWithWebIdentity",
+            "Condition": {
+                "StringLike": {
+                    f"{context.eks_oidc_provider}:sub": "system:serviceaccount:kube-system:fsx-csi-controller-sa"
+                }
+            },
+        },
+    )
+
+    iam.add_assume_role_statement(
+        role_name=f"orbit-{context.name}-admin",
+        statement={
+            "Effect": "Allow",
+            "Principal": {"Federated": f"arn:aws:iam::{context.account_id}:oidc-provider/{context.eks_oidc_provider}"},
+            "Action": "sts:AssumeRoleWithWebIdentity",
+            "Condition": {
+                "StringLike": {
+                    f"{context.eks_oidc_provider}:sub": f"system:serviceaccount:orbit-system:orbit-{context.name}-admin"
+                }
+            },
+        },
+    )
+
     _logger.debug("EKSCTL deployed")
 
 
@@ -435,6 +469,8 @@ def deploy_team(context: "Context", team_context: "TeamContext") -> None:
             )
             subnets_ids = [s.subnet_id for s in subnets]
 
+            sh.run(f"aws eks wait cluster-active --name orbit-{context.name} --debug")
+
             eks.create_fargate_profile(
                 profile_name=f"orbit-{context.name}-{team_context.name}",
                 cluster_name=f"orbit-{context.name}",
