@@ -16,7 +16,7 @@ import logging
 import time
 from datetime import datetime, timezone
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Dict, Iterable, List, NamedTuple, Optional, Union, cast
+from typing import TYPE_CHECKING, Any, Dict, Iterable, List, NamedTuple, Optional, Union
 
 import botocore.exceptions
 import yaml
@@ -33,7 +33,7 @@ _logger: logging.Logger = logging.getLogger(__name__)
 
 _BUILD_WAIT_POLLING_DELAY: float = 5  # SECONDS
 
-CDK_VERSION = "~=1.67.0"
+CDK_VERSION = "~=1.100.0"
 CDK_MODULES = [
     "aws_cdk.core",
     "aws-cdk.aws-ec2",
@@ -127,10 +127,11 @@ def start(
     client = boto3_client("codebuild")
     repo: Optional[str] = None
     credentials: Optional[str] = None
-    if context.images.code_build.source and context.images.code_build.source.startswith("ecr"):
-        repo = cast(str, context.images.code_build.repository)
+    if context.images.code_build.get_source(account_id=context.account_id, region=context.region).startswith("ecr"):
+        version = context.images.code_build.version
+        repo = f"{context.images.code_build.repository}:{version}"
         if not any(match in repo for match in [".amazonaws.com/", "public.ecr.aws"]):
-            repo = f"{context.account_id}.dkr.ecr.{context.region}.amazonaws.com/{repo}"
+            repo = f"{context.account_id}.dkr.ecr.{context.region}.amazonaws.com/{repo}:{version}"
         credentials = "SERVICE_ROLE"
     _logger.debug("Repository: %s", repo)
     _logger.debug("Credentials: %s", credentials)
@@ -255,7 +256,10 @@ def generate_spec(
 
     install.append("pwd")
     # Orbit Workbench CLI
-    install.append(f"pip install aws-orbit~={__version__}")
+    if __version__.endswith(".dev0"):
+        install.append(f"pip install aws-orbit=={__version__}")
+    else:
+        install.append(f"pip install aws-orbit~={__version__}")
 
     # Plugins
     if plugins and isinstance(context, Context):
